import re
from . import helper, preprocessor, postprocessor, fixer
from functools import cmp_to_key

class Transliterator:
    def __init__(self, src, tgt, preoptions=[], postoptions=[]):

        self.src = src
        self.tgt = tgt
        self.postoptions = postoptions
        self.preoptions = preoptions

        self.preprocessor = preprocessor.Preprocessor(self.src, self.tgt, self.preoptions, self.postoptions)
        self.postprocessor = postprocessor.PostProcessor(self.src, self.tgt, self.preoptions, self.postoptions)
        self.fixer = fixer.Fixer(self.src, self.tgt, self.preoptions, self.postoptions)

    def convert(self, txt):
        txt = self.preprocessor.preprocess(txt)
        transliteration = self.convertScript(txt)
        transliteration = self.postprocessor.postprocess(transliteration)

        return transliteration

    def convertInter(self, Strng):
        ScriptAll = helper.Vowels+helper.Consonants+helper.CombiningSigns+helper.Numerals+helper.Signs+helper.Aytham
        SourceScript = helper.CrunchSymbols(ScriptAll,self.src)
        TargetScript = helper.CrunchSymbols(ScriptAll,helper.Inter)
        ScriptMapAll = sorted(zip(SourceScript,TargetScript),key=cmp_to_key(helper.lenSort))

        for x,y in ScriptMapAll:
            Strng = Strng.replace(x,y)

        return Strng

    # Conversion Module
    def convertScript(self, Strng):
        charPairs=[];
        Schwa = '\uF000'
        DepV = '\u1E7F'

        Strng = self.convertInter(Strng)
        Source = helper.Inter
        Strng = self.indianize_inter(Strng)

        Strng = Strng.replace("ṿ×_","ṿ")
        Strng = Strng.replace("ṿ×_","ṿ")

        ha = helper.CrunchSymbols(helper.Consonants,Source)[32]
        charPairs=[]

        for charList in helper.ScriptAll:
            # Crunch all related characters into a list
            TargetScript = helper.CrunchSymbols(helper.retCharList(charList),self.tgt)
            if charList == 'VowelSigns':
                # Add DepVSign to all VowelSigns to differentiate from Independent Vowels
                SourceScript = [DepV+x for x in helper.CrunchSymbols(helper.VowelSigns,Source)]
            else:
                SourceScript = helper.CrunchSymbols(helper.retCharList(charList),Source)
            # Create a Tuple for the conversion pair
            ScriptMap = list(zip(SourceScript,TargetScript))
            # Sort the mapping in descending order. Longer Characters are to be replaced first. ऍˇ > ऍ
            ScriptMap.sort(reverse=True);
            charPairs= charPairs + ScriptMap

        charPairs = sorted(charPairs,key=cmp_to_key(helper.lenSort))

        # Perform replacement sequentially for each character group
        for x,y in charPairs:
            #print(x, '-->', y)
            Strng = Strng.replace(x,y)
            #print(Strng)

        ## a_i => a<dev>i<dev> ; a_u = a<dev>u<dev>
        Strng=Strng.replace('_' + helper.CrunchSymbols(helper.Vowels,self.tgt)[2],  helper.CrunchSymbols(helper.Vowels,self.tgt)[2])
        Strng=Strng.replace('_' + helper.CrunchSymbols(helper.Vowels,self.tgt)[4],  helper.CrunchSymbols(helper.Vowels,self.tgt)[4])

        ## Joiners Vir + ZWJ
        vir = helper.CrunchList('ViramaMap', self.tgt)[0]
        Strng = Strng.replace(vir + "[]", "\u200D" + vir)

        if Source in ['Inter']:
            Strng = Strng.replace('\u00D7', vir) # special explicit Virama

        # Apply Fixes on the Output based on the Script
        Strng = self.fixer.fix(Strng)

        Strng = self.remove_temp_markers(Strng)

        return Strng

    ## This is the processing that's done immediately after calling the convert() function
    ## But before the post-processing steps
    ## removes extranous/artificial diacritics introduced
    def remove_temp_markers(self, Strng):
        Strng = Strng.replace("\uF001", "").replace("\u05CC", "").\
            replace("ʻʻ", "").replace('\u05CD', '').replace('\u02C2', '') ## remove token characters for specialized processing

        return Strng

    def indianize_inter(self, Strng):
        DepV = '\u1E7F'
        Asp = '\u02B0'

        Vir = helper.CrunchList("ViramaMap", 'Inter')[0]
        Nuk = helper.CrunchList("NuktaMap", 'Inter')[0]
        VowelA = helper.CrunchSymbols(['VowelMap'], 'Inter')[0]

        ListV = '|'.join(helper.CrunchSymbols(helper.VowelSigns, 'Inter'))
        ListC = '|'.join(helper.CrunchSymbols(helper.Consonants, 'Inter'))

        Strng = re.sub('('+ListC+')'+'(?!'+ListV+'|'+VowelA+')',r'\1'+DepV+Vir,Strng)

        Strng = re.sub('('+ListC+'|'+Nuk+')'+'('+ListV+')',r'\1'+DepV+r'\2',Strng)

        Strng = re.sub('(?<='+ListC+')'+'('+VowelA+')',r'',Strng)

        return Strng




# -*- coding: utf-8 -*-

import re
from . import helper

class Preprocessor:
    def __init__(self, src, tgt, preoptions=[], postoptions=[], defaults=True):
        self.src = src
        self.tgt = tgt
        self.postoptions = postoptions
        self.preoptions = preoptions
        self.defaults = defaults

    def preprocess(self, Strng):
        Strng = self.default_preprocess(Strng)

        try:
            for option in self.preoptions:
                Strng = getattr(self, option)(Strng)
        except Exception as e:
            pass
            #print(e)

        return Strng

    def default_preprocess(self, Strng):
        if self.tgt in helper.IndicScripts:
            Strng = self.ipa_indic_preprocess(Strng)

        return Strng

    def ipa_indic_preprocess(self, Strng):
        ties = [
                #('ʊ','u'),
                ('eɪ', 'e͡ɪ'), ('ou', 'o͡u'), ('oʊ', 'o͡ʊ'), ('aɪ', 'a\u0361ɪ'), ('aʊ','a\u0361ʊ'),
                ('əʊ', 'ə\u0361ʊ')
                ]

        #introduce ties for Tamil for Dipthongs
        for x, y in ties:
            Strng = Strng.replace(x, y)

        #remove the ties if not Tamil or IPA
        if 'TamilPhonetic' not in self.tgt and 'IPA' not in self.tgt:
            Strng = Strng.replace('\u0361', '')

        # move stress mark before the consonant p'o -> 'po
        C = '|'.join(helper.CrunchSymbols(helper.Consonants,'IPA'))
        Diac = '|'.join(['ˈ', 'ˌ'])

        Strng = re.sub('(('+C+')+)'+'('+Diac+')',r'\3\1',Strng)

        return Strng

    def monophthongize(self, Strng):
        eRep = [('eɪ', 'eː') ]
        oRep = [('əʊ', 'oː'), ('oʊ','oː')]
        aiRep = [('aɪ', 'əɪ')]
        auRep = [('aʊ', 'əʊ')]

        if 'Devanagari' in self.tgt and 'High' not in self.tgt:
            patterns = eRep + oRep
        else:
            patterns = eRep + oRep + aiRep + auRep

        for x, y in patterns:
            # if the tie optionally appears
            Strng = re.sub(x[0]+'\u0361?'+x[1],y, Strng)

        return Strng




from . import helper
from uchcharaka.ScriptMap.MainIndic import Urdu, Malayalam,Sinhala
import re

class Fixer:
    def __init__(self, src, tgt, preoptions=[], postptions=[]):
        self.src = src
        self.tgt = tgt
        self.postoptions = preoptions
        self.preoptions = postptions

    # Fixing the Indic Ouput for the standard corrections
    # Indic Fix are mandatory corrections to the immediate ouput.
    def fix(self, Strng):
        vir = helper.CrunchList('ViramaMap', self.tgt)[0]

        Strng = Strng.replace(vir + '_', vir)

        try:
            fixerFunction = "Fix"+self.tgt
            for suffix in helper.suffixes:
                fixerFunction = fixerFunction.replace(suffix, '')
            Strng = getattr(self, fixerFunction)(Strng)
        except Exception as e:
            pass
            #print(e)

        Strng = Strng.replace('\u02BD','')

        return Strng

    def FixKannada(self, Strng):
        #fix Transcribe transcription
        Strng = Strng.replace('\u0CA8\u0CCD\u0CB8\u0CCD\u0C95\u0CCD\u0CB0', \
                              '\u0CA8\u0CCD\u0CB8\u0CCD\u200C\u0C95\u0CCD\u0CB0')

        #fix scry
        Strng = Strng.replace('\u0CCD\u0C95\u0CCD\u0CB0\u0CC8', '\u0CCD\u200C\u0C95\u0CCD\u0CB0\u0CC8')

        return Strng

    # Urdu - Shadda, Final E
    def FixUrdu(self, Strng):
        # .replace(u'\u064E','')

        #print('I am here in Fixing Urdu')

        Strng = Strng.replace('\u02BD','')

        vir = helper.CrunchSymbols(helper.VowelSigns,self.tgt)[0]

        ConUnAsp = [helper.CrunchList('ConsonantMap', self.tgt)[x] for x in [0,2,5,7,10,12,15,17,20,22,4,9,14,19,24]+list(range(25,33))] # Add SemiVowels & Nukta Consonants
        ConUnAsp = ConUnAsp + helper.CrunchList('SouthConsonantMap',self.tgt) + helper.CrunchList('NuktaConsonantMap',self.tgt)

        ## Add word-final E, Short A sign

        ShortVowels = '|'.join(['\u0652','\u064E','\u0650','\u064F'])
        a = '\u064E'
        ya = '\u06CC'
        va  = '\u0648'
        yaBig = '\u06D2'
        Aa = Urdu.VowelSignMap[0]

        #Strng = Strng.replace(u'\u064E'+ya,ya)
        #Strng = Strng.replace(u'\u064E'+va,va)

        ## Fixing Hamza

        ListVS = '(' + '|'.join(helper.CrunchSymbols(helper.VowelSigns,self.tgt)) + ')'
        ListV = '(' + '|'.join(helper.CrunchSymbols(helper.Vowels,self.tgt)) + ')'
        ListVSA = '(' + '|'.join(helper.CrunchSymbols(helper.VowelSigns,self.tgt)+[a]) + ')'

        ListC = '(' + '|'.join(helper.CrunchSymbols(helper.Consonants,self.tgt)) + ')'
        # Fix NunGhunna
        Strng = re.sub('(\u06BA\u02BD?)' + ListC, '\u0646\u0658' + r'\2', Strng)

        hamzaFull = "\u0621"
        hamzaChair = "\u0626"

        Strng = re.sub(ListVS + ListV, r'\1' + hamzaFull + r'\2', Strng)
        Strng = re.sub(ListV + ListV, r'\1' + hamzaFull + r'\2', Strng)
        Strng = re.sub('('+a+')' + ListV +'(?!' + ListVSA + ')'   , r'\1' + hamzaFull + r'\2', Strng)

        Strng = re.sub('('+a+')'+'('+ShortVowels+')',r'\2',Strng)
        Strng = re.sub('(?<!'+Aa+')'+'('+a+')'+'('+va+'|'+ya+')'+'(?!'+ ShortVowels +')',r'\2',Strng)
        ListC = '|'.join(helper.CrunchSymbols(helper.Consonants,self.tgt)).replace(a,'')
        Ayoga = '|'.join(Urdu.AyogavahaMap[0] + Urdu.AyogavahaMap[1])

        Strng = Strng.replace(ya,yaBig)
        Strng = re.sub('('+yaBig+')'+'(?='+'|'.join(ConUnAsp)+ShortVowels+')',ya,Strng)
        Strng = re.sub('('+yaBig+')'+'('+ListC+')',ya+r'\2',Strng)
        Strng = re.sub('('+yaBig+')'+'('+Ayoga+')',ya+r'\2',Strng)

        Strng = Strng.replace('\u0650'+yaBig,'\u0650'+ya)

        #Strng = Strng.replace(a+Urdu.VowelSignMap[0],Urdu.VowelSignMap[0]) ## remove a sign from consonants

        ## ye ## yezu ## Fix this

        ## Adding Gemination of Consonant

        ConAsp   = [helper.CrunchList('ConsonantMap', self.tgt)[x] for x in [1,3,6,8,11,13,16,18,21,23]]
        ConUnAsp_a =  [x.replace('\u064e','') for x in ConUnAsp]

        Strng = re.sub('('+'|'.join(ConUnAsp_a)+')'+'('+vir+')'+r'\1',r'\1'+helper.Gemination[self.tgt],Strng)

        ## Move Shadda to the ha do=chasmee
        ## katthu -> کَتھُّ
        Strng = re.sub('(.)(ّ)(\u06BE)', r'\1\3\2', Strng)

        ## Fix

        Strng = Strng.replace('ےے', 'یے')
        Strng = Strng.replace('ےی', 'یی')
        Strng = Strng.replace('ےْ', 'یْ')

        #http://www.columbia.edu/~mk2580/urdu_section/handouts/hamza.pdf

        # Hamza on ya
        Strng = Strng.replace('ءاِی','\u0626\u0650\u06CC')
        Strng = Strng.replace('ءاے', 'ئے')

        Strng = Strng.replace('ءای', 'ئی')

        # Hamza on waw
        Strng = Strng.replace('ءاو','ؤ') #o
        Strng = Strng.replace('ءاُو', '\u0624\u064F') #long u

        Strng = Strng.replace('ءاُ', '\u0624\u064F') #short u

        # kau/ kaU

        Strng = re.sub('('+hamzaFull+')(اُو)', r'\1'+'\u0624\u064F', Strng)
        Strng = re.sub('('+hamzaFull+')(اُ)', r'\1'+'\u0624\u064F', Strng)

        Strng = re.sub('('+hamzaFull+')(او)', r'\1'+'\u0624', Strng)

        # Hamza on ya for short i
        Strng = Strng.replace('ءاِ', '\u0626\u0650')
        Strng = Strng.replace('ئِءآ', '\u0626\u0650\u0627')

        Strng = re.sub('('+hamzaFull+')(\u0627\u0650)', r'\1'+'\u0626\u0650', Strng)

        # for gae kae etc
        Strng = re.sub('('+hamzaFull+')(ا)(ے|ی)', r'\1'+'\u0626' + r'\3', Strng)

        ## Fix Double Hamza to Single Hamza

        Strng = Strng.replace('ئِئ', 'ئِ')
        Strng = Strng.replace('ئِؤ', 'ئِو')

        ## Fix Retroflex with combining characters
        Strng = Strng.replace('ࣇ', 'لؕ')

        if self.tgt == 'Shahmukhi':
            #aspirated nasals and liquids
            Strng = re.sub('(ن|م|ی|ر|ل|و)(\u0652)(ہ)',r'\1'+'\u06BE', Strng)

        # Fix word-final ha

        Strng = self.PersoArabicPuntuation(Strng)

        return Strng

    def PersoArabicPuntuation(self, Strng):
        # Punctuation
        for x,y in zip([',','?',';'],['،','؟','؛']):
            Strng = Strng.replace(x,y)
        Strng = Strng.replace(".", "۔")

        return Strng

    def FixSinhala(self, Strng):
        Strng = self.SinhalaDefaultConjuncts(Strng)

        Strng = Strng.replace("\u0DA2\u0DCA\u0DA4","\u0DA5")
        Strng = Strng.replace("(අ)(අ)","(ආ)")

        return Strng

    def FixMalayalam(self, Strng):
        Strng = self.MalayalamChillu(Strng)

        vir = Malayalam.ViramaMap[0]
        Strng = re.sub('('+vir+')'+ '(ര̣)', r'\1' + '\u200C' + r'\2', Strng)

        return Strng

    def SinhalaDefaultConjuncts(self, Strng):
        vir = Sinhala.ViramaMap[0]
        YR = '|'.join(Sinhala.ConsonantMap[25:27])

        Strng = re.sub('('+vir+')'+'('+YR+')',r'\1'+'\u200D'+r'\2',Strng)
        Strng = re.sub('('+YR[2]+')'+'('+vir+')'+'('+'\u200D'+')'+'('+YR[0]+')',r'\1\3\2\3\4',Strng)

        Strng = Strng.replace(Sinhala.ConsonantMap[7]+Sinhala.ViramaMap[0]+Sinhala.ConsonantMap[9],'\u0DA5')
        Strng = Strng.replace(Sinhala.ConsonantMap[0]+vir+Sinhala.ConsonantMap[30],Sinhala.ConsonantMap[0]+vir+'\u200D'+Sinhala.ConsonantMap[30])

        ## KSHA

        Strng = Strng.replace('ර‍්‍ය', 'ර්ය')
        Strng = Strng.replace('ර්‍ර', 'ර්ර')

        return Strng

    def MalayalamChillu(self, Strng):
        Chillus=['\u0D7A','\u0D7B','\u0D7C','\u0D7D','\u0D7E', 'ഩ‍്']

        ListC = '(' + '|'.join(helper.CrunchSymbols(helper.CharactersNV,self.tgt) + ['ഽ']) + ')'

        vir = Malayalam.ViramaMap[0]
        ConVir =[
                Malayalam.ConsonantMap[14]+vir,
                Malayalam.ConsonantMap[19]+vir,
                Malayalam.ConsonantMap[26]+vir,
                Malayalam.ConsonantMap[27]+vir,
                Malayalam.SouthConsonantMap[0]+vir,
                'ഩ്'
                ]

        ## may be include ha ?
        CList = [
                Malayalam.ConsonantMap[10:15]+Malayalam.ConsonantMap[24:26]+Malayalam.ConsonantMap[28:29],
                Malayalam.ConsonantMap[15:20]+Malayalam.ConsonantMap[24:27]+Malayalam.ConsonantMap[28:29]+['റ'],
                Malayalam.ConsonantMap[25:27],
                Malayalam.ConsonantMap[20:21] + Malayalam.ConsonantMap[24:26] + Malayalam.ConsonantMap[27:29],
                Malayalam.SouthConsonantMap[0:1]+Malayalam.ConsonantMap[25:27],
                Malayalam.ConsonantMap[15:20]+Malayalam.ConsonantMap[24:27]+Malayalam.ConsonantMap[28:29]+['റ']
                ]

        for i in range(len(Chillus)):
            #print '(?<!'+'['+vir+''.join(Chillus)+']'+')'+'('+ConVir[i]+')'+'(?!['+''.join(CList[i])+'])'
            Strng = re.sub(ListC + helper.VedicSvaras + '('+ConVir[i]+')'+'(?!['+''.join(CList[i])+'])',r'\1\2' + Chillus[i],Strng)
            Strng = re.sub(ListC + helper.VedicSvaras + '('+ConVir[i]+')'+'(?=(['+''.join(CList[i])+'])' + vir + r'\4' + ')',r'\1\2' + Chillus[i],Strng)

        ## remove _ appearing due to the preserve chillu option
        Strng = re.sub('(?<!ത്)ˍ', '', Strng)

        return Strng


import sys,os, json, pytest
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "../..")))

from uchcharaka import transcriber as tr

languages = ['it', 'es-419', 'es', 'en-us', 'en-gb', 'de', 'fr', 'pt-br', 'pt-pt']
scripts = ['Tamil', 'Telugu', 'Kannada', 'Urdu', 'Devanagari', 'IPA', 'Sinhala', 'Malayalam']
accuracy = ['Low', 'High', 'Medium']
src_texts = {
        'it': "L'italiano è una lingua romanza parlata principalmente in Italia. Per ragioni storiche e geografiche, l'italiano è la lingua romanza meno divergente dal latino (complessivamente a pari merito, anche se in parametri diversi, con la lingua sarda). L'italiano è classificato al 23º posto tra le lingue per numero di parlanti nel mondo e, in Italia, è utilizzato da circa 58 milioni di residenti. Nel 2015 era la lingua materna del 90,4% dei residenti in Italia, che spesso lo acquisiscono e lo usano insieme alle varianti regionali dell'italiano, alle lingue regionali e ai dialetti. In Italia viene ampiamente usato per tutti i tipi di comunicazione della vita quotidiana e prevale largamente nei mezzi di comunicazione nazionali, nell'amministrazione pubblica dello Stato italiano e nell'editoria. Oltre ad essere la lingua ufficiale dell'Italia e di San Marino, è anche una delle lingue ufficiali dell'Unione europea, della Svizzera, della Città del Vaticano e del Sovrano militare ordine di Malta. È inoltre riconosciuto e tutelato come «lingua della minoranza nazionale italiana» dalla Costituzione slovena e croata nei territori in cui vivono popolazioni di dialetto istriano. È diffuso nelle comunità di emigrazione italiana, è ampiamente noto anche per ragioni pratiche in diverse aree geografiche ed è una delle lingue straniere più studiate nel mondo.Dal punto di vista storico, l'italiano è una lingua codificata tra Quattrocento e Cinquecento sulla base del fiorentino letterario usato nel Trecento.",
        'pt-br': 'A língua portuguesa, também designada português, é uma língua indo-europeia românica flexiva ocidental originada no galego-português falado no Reino da Galiza e no norte de Portugal. Com a criação do Reino de Portugal em 1139 e a expansão para o sul na sequência da Reconquista, deu-se a difusão da língua pelas terras conquistadas e, mais tarde, com as descobertas portuguesas, para o Brasil, África e outras partes do mundo. O português foi usado, naquela época, não somente nas cidades conquistadas pelos portugueses, mas também por muitos governantes locais nos seus contatos com outros estrangeiros poderosos. Especialmente nessa altura, a língua portuguesa também influenciou várias línguas. Durante a Era dos Descobrimentos, marinheiros portugueses levaram o seu idioma para lugares distantes. A exploração foi seguida por tentativas de colonizar novas terras para o Império Português e, como resultado, o português dispersou-se pelo mundo. Brasil e Portugal são os dois únicos países cuja língua primária é o português. É língua oficial em antigas colônias portuguesas, nomeadamente, Moçambique, Angola, Cabo Verde, Guiné Equatorial, Guiné-Bissau e São Tomé e Príncipe, todas na África. Além disso, por razões históricas, falantes do português, ou de crioulos portugueses, são encontrados também em Macau (China), Timor-Leste, em Damão e Diu, nos estados de Goa (Índia) e Malaca (na Malásia, que conta com utilizadores do crioulo kristáng, ou Língua cristã), em enclaves na ilha das Flores (Indonésia), Baticaloa no (Sri Lanka) e nas ilhas ABC no Caribe.',
        'pt-pt': 'A língua portuguesa, também designada português, é uma língua indo-europeia românica flexiva ocidental originada no galego-português falado no Reino da Galiza e no norte de Portugal. Com a criação do Reino de Portugal em 1139 e a expansão para o sul na sequência da Reconquista, deu-se a difusão da língua pelas terras conquistadas e, mais tarde, com as descobertas portuguesas, para o Brasil, África e outras partes do mundo. O português foi usado, naquela época, não somente nas cidades conquistadas pelos portugueses, mas também por muitos governantes locais nos seus contatos com outros estrangeiros poderosos. Especialmente nessa altura, a língua portuguesa também influenciou várias línguas. Durante a Era dos Descobrimentos, marinheiros portugueses levaram o seu idioma para lugares distantes. A exploração foi seguida por tentativas de colonizar novas terras para o Império Português e, como resultado, o português dispersou-se pelo mundo. Brasil e Portugal são os dois únicos países cuja língua primária é o português. É língua oficial em antigas colônias portuguesas, nomeadamente, Moçambique, Angola, Cabo Verde, Guiné Equatorial, Guiné-Bissau e São Tomé e Príncipe, todas na África. Além disso, por razões históricas, falantes do português, ou de crioulos portugueses, são encontrados também em Macau (China), Timor-Leste, em Damão e Diu, nos estados de Goa (Índia) e Malaca (na Malásia, que conta com utilizadores do crioulo kristáng, ou Língua cristã), em enclaves na ilha das Flores (Indonésia), Baticaloa no (Sri Lanka) e nas ilhas ABC no Caribe.',
        'de': 'Die deutsche Sprache - Deutsch ist eine westgermanische Sprache, die weltweit etwa 90 bis 105 Millionen Menschen als Muttersprache und weiteren rund 80 Millionen als Zweitsprache oder Fremdsprache dient. Das Deutsche ist eine plurizentrische Sprache, enthält also mehrere Standardvarietäten in verschiedenen Regionen. Ihr Sprachgebiet umfasst Deutschland, Österreich, die Deutschschweiz, Liechtenstein, Luxemburg, Ostbelgien, Südtirol, das Elsass und den Nordosten Lothringens sowie Nordschleswig. Außerdem ist Deutsch eine Minderheitensprache in einigen europäischen und außereuropäischen Ländern, zum Beispiel in Rumänien und Nationalsprache im afrikanischen Namibia. Deutsch ist die meistgesprochene Muttersprache in der Europäischen Union (EU). Ursprünglich bestand eine Vielzahl von Mundarten innerhalb eines Dialektkontinuums, das sich aufgrund der zweiten (hochdeutschen) Lautverschiebung in hochdeutsche (oberdeutsche und mitteldeutsche) und niederdeutsche Mundarten einteilen lässt. Da die Niederdeutsche Sprache außerhalb des Hochdeutschen steht, ist sie oft nicht mitgemeint, wenn von „der deutschen Sprache“ die Rede ist. Teilweise wird Niederdeutsch - „Platt“ aber auch als Dialekt des Deutschen behandelt. Die deutsche Standardsprache ist mit ihren Standardvarietäten bundesdeutsches Deutsch, österreichisches Deutsch und schweizerisches Deutsch das Ergebnis bewusster sprachplanerischer Eingriffe. Das Standarddeutsche überspannt als Dachsprache den Großteil der Mundarten des Dialektkontinuums. Eine Ausnahme sind zum Beispiel die Luxemburger Dialekte, die nunmehr unter Letzebuergesch und somit nur noch indirekt als Deutsch zusammengefasst werden.',
        'fr': "Le français est parlé, en 2024, sur tous les continents par environ 343 millions de personnes: 235 millions l'emploient quotidiennement et 113 millions en sont des locuteurs natifs. En 2018, 80 millions d'élèves et étudiants s'instruisent en français dans le monde. Selon l'Organisation internationale de la francophonie (OIF), il pourrait y avoir 700 millions de francophones sur Terre en 2050. Le français est la cinquième langue parlée au monde après l'anglais, le mandarin, le hindi et l'espagnol. Elle est également la deuxième langue apprise sur le globe et la troisième langue des affaires et du commerce. Le français se classe deuxième parmi les langues étrangères les plus fréquemment enseignées à travers le monde. Il est également la quatrième langue utilisée sur Internet après l'espagnol, le mandarin et l'anglais, langue dont le vocabulaire a été fortement enrichi par le français. Dans le monde, vingt-sept États ont le français comme langue officielle. C'est une des six langues officielles ainsi qu'une des deux langues de travail de l'Organisation des Nations unies. Le français est une langue officielle ou de travail de nombreuses organisations gouvernementales internationales, parmi lesquelles l'Union postale universelle ou les trois autorités mondiales de régulation du système métrique. Il est aussi langue officielle ou de travail de nombreuses organisations gouvernementales régionales, telles que l'Union africaine ou l’Union européenne, et est aussi langue officielle ou de travail de nombreuses organisations non gouvernementales internationales, comme le Comité international olympique ou le Mouvement international de la Croix-Rouge et du Croissant-Rouge. Grâce à sa présence sur tous les continents et du fait qu'elle est l'une des langues officielles de l'ONU ainsi qu'une de ses langues de travail, le français figure parmi les langues les plus influentes du monde. ",
        'es-419': 'El español o castellano es una lengua romance procedente del latín hablado, perteneciente a la familia de lenguas indoeuropeas. Forma parte del grupo ibérico y es originaria de Castilla, reino medieval de la península ibérica. Hay más de 600 millones de personas que hablan español, de las cuales casi 500 millones son hablantes nativos. Es el idioma más estudiado en todos los niveles de enseñanza de Estados Unidos y la segunda lengua más estudiada en la educación secundaria superior de la Unión Europea.El español es la cuarta lengua más estudiada del mundo con 24,2 millones de alumnos. Es la cuarta lengua más hablada del mundo (tras el inglés, chino mandarín e hindi) y la segunda lengua más hablada por número de hablantes nativos (tras el chino mandarín). Así pues, puede ser considerado como el segundo idioma en comunicación internacional (tras el inglés).El español tiene la tercera población alfabetizada del mundo. En 2008, el español tenía el 5,47 % de la población mundial alfabetizada. El español es la tercera lengua más utilizada para la producción de información en los medios de comunicación, como también la tercera lengua con más usuarios de Internet, después del chino y el inglés. En 2019, el español tenía unos 364 millones de usuarios en Internet, lo cual representaba el 7,9 % del total. La lengua es hablada principalmente en España e Hispanoamérica. En 2015, se estimó que en los Estados Unidos había más de 40 millones de hispanohablantes.En Zamboanga, donde el español ya no es lengua de habla mayoritaria ni oficial, el mismo sigue manteniendo importancia en el sentido cultural, histórico y lingüístico. En Andorra, la mayoría de la población sabe hablar español. Guinea Ecuatorial, donde es lengua oficial, alrededor del 70 % de su población sabe hablar español. En los campos de refugiados de la República Árabe Saharaui Democrática el español es un idioma significativo.',
        'es': 'El español o castellano es una lengua romance procedente del latín hablado, perteneciente a la familia de lenguas indoeuropeas. Forma parte del grupo ibérico y es originaria de Castilla, reino medieval de la península ibérica. Hay más de 600 millones de personas que hablan español, de las cuales casi 500 millones son hablantes nativos. Es el idioma más estudiado en todos los niveles de enseñanza de Estados Unidos y la segunda lengua más estudiada en la educación secundaria superior de la Unión Europea.El español es la cuarta lengua más estudiada del mundo con 24,2 millones de alumnos. Es la cuarta lengua más hablada del mundo (tras el inglés, chino mandarín e hindi) y la segunda lengua más hablada por número de hablantes nativos (tras el chino mandarín). Así pues, puede ser considerado como el segundo idioma en comunicación internacional (tras el inglés).El español tiene la tercera población alfabetizada del mundo. En 2008, el español tenía el 5,47 % de la población mundial alfabetizada. El español es la tercera lengua más utilizada para la producción de información en los medios de comunicación, como también la tercera lengua con más usuarios de Internet, después del chino y el inglés. En 2019, el español tenía unos 364 millones de usuarios en Internet, lo cual representaba el 7,9 % del total. La lengua es hablada principalmente en España e Hispanoamérica. En 2015, se estimó que en los Estados Unidos había más de 40 millones de hispanohablantes.En Zamboanga, donde el español ya no es lengua de habla mayoritaria ni oficial, el mismo sigue manteniendo importancia en el sentido cultural, histórico y lingüístico. En Andorra, la mayoría de la población sabe hablar español. Guinea Ecuatorial, donde es lengua oficial, alrededor del 70 % de su población sabe hablar español. En los campos de refugiados de la República Árabe Saharaui Democrática el español es un idioma significativo.',
        'en-us': 'English is a West Germanic language in the Indo-European language family, whose speakers, called Anglophones, originated in early medieval England on the island of Great Britain. The namesake of the language is the Angles, one of the ancient Germanic peoples that migrated to Britain. It is the most spoken language in the world, primarily due to the global influences of the former British Empire (succeeded by the Commonwealth of Nations) and the United States. English is the third-most spoken native language, after Mandarin Chinese and Spanish; it is also the most widely learned second language in the world, with more second-language speakers than native speakers. English is either the official language or one of the official languages in 59 sovereign states (such as India, Ireland, and Canada). In some other countries, it is the sole or dominant language for historical reasons without being explicitly defined by law (such as in the United States and United Kingdom). It is a co-official language of the United Nations, the European Union, and many other international and regional organisations. It has also become the de facto lingua franca of diplomacy, science, technology, international trade, logistics, tourism, aviation, entertainment, and the Internet. English accounts for at least 70% of total speakers of the Germanic language branch, and as of 2021, Ethnologue estimated that there were over 1.5 billion speakers worldwide.Old English emerged from a group of West Germanic dialects spoken by the Anglo-Saxons. Late Old English borrowed some grammar and core vocabulary from Old Norse, a North Germanic language. Then, Middle English borrowed vocabulary extensively from French dialects, which are the source of approximately 28% of Modern English words, and from Latin, which is the source of an additional 28%. As such, though most of its total vocabulary comes from Romance languages, Modern English\'s grammar, phonology, and most commonly used words in everyday use keep it genealogically classified under the Germanic branch. It exists on a dialect continuum with Scots and is then most closely related to the Low Saxon and Frisian languages.',
        'en-gb': 'English is a West Germanic language in the Indo-European language family, whose speakers, called Anglophones, originated in early medieval England on the island of Great Britain. The namesake of the language is the Angles, one of the ancient Germanic peoples that migrated to Britain. It is the most spoken language in the world, primarily due to the global influences of the former British Empire (succeeded by the Commonwealth of Nations) and the United States. English is the third-most spoken native language, after Mandarin Chinese and Spanish; it is also the most widely learned second language in the world, with more second-language speakers than native speakers. English is either the official language or one of the official languages in 59 sovereign states (such as India, Ireland, and Canada). In some other countries, it is the sole or dominant language for historical reasons without being explicitly defined by law (such as in the United States and United Kingdom). It is a co-official language of the United Nations, the European Union, and many other international and regional organisations. It has also become the de facto lingua franca of diplomacy, science, technology, international trade, logistics, tourism, aviation, entertainment, and the Internet. English accounts for at least 70% of total speakers of the Germanic language branch, and as of 2021, Ethnologue estimated that there were over 1.5 billion speakers worldwide.Old English emerged from a group of West Germanic dialects spoken by the Anglo-Saxons. Late Old English borrowed some grammar and core vocabulary from Old Norse, a North Germanic language. Then, Middle English borrowed vocabulary extensively from French dialects, which are the source of approximately 28% of Modern English words, and from Latin, which is the source of an additional 28%. As such, though most of its total vocabulary comes from Romance languages, Modern English\'s grammar, phonology, and most commonly used words in everyday use keep it genealogically classified under the Germanic branch. It exists on a dialect continuum with Scots and is then most closely related to the Low Saxon and Frisian languages.'
      }

# Read JSON file and convert it back to a dictionary
with open("texts.json", "r", encoding="utf-8") as json_file:
    texts = json.load(json_file)  # Convert JSON to dictionary

class TestOverall:
    @pytest.mark.parametrize("language", languages)
    @pytest.mark.parametrize("script", scripts)
    @pytest.mark.parametrize("accuracy", accuracy)
    def test_basic(self, language, script, accuracy):
        transcriber = tr.Transcriber(language, script, accuracy=accuracy, preoptions = [], postoptions = [])
        assert texts[language][script][accuracy] == transcriber.transcribe(src_texts[language])


import re
from . import helper
import espeak_py as espeak
from . import transliterator as tr

class Transcriber:
    def __init__(self, src, tgt, accuracy='medium', preoptions=[], postoptions=[]):
        self.src = src

        scriptList = helper.IndicScripts + helper.LatinScripts
        scriptListLower = list(map(lambda x: x.lower(), scriptList))

        # speical processing for Tamil
        if 'removeDifferentiation' in postoptions and 'Tamil' in tgt:
            accuracy = 'Low'

        #Tamil Medium --> TamilMedium
        if tgt not in helper.inaccurate_scripts:
            tgt = tgt + accuracy.title()

        if tgt.lower() in scriptListLower:
            self.tgt = [script_id for script_id in scriptList if tgt.lower() == script_id.lower()][0]
        else:
            raise Exception(f"Target script: {tgt} not supported")

        self.accuracy = accuracy.lower()

        #introduce postprocessing for non-accurate transcriptions
        if self.accuracy != 'high':
            postoptions = postoptions + [self.tgt]

        self.postoptions = postoptions

        # simplify Dipththongs in English for non-accurate transcriptions
        if (self.accuracy != 'high' or self.tgt in helper.inaccurate_scripts)\
            and 'en' in self.src and 'showDiphthongs' not in self.postoptions:
            preoptions = ['monophthongize'] + preoptions

        self.preoptions = preoptions

    def transcribe(self, Strng):
        # transcribe to IPA only if source is not IPA
        if self.src != 'IPA':
            try:
                ipa_transcription = espeak.text_to_phonemes(Strng, self.src)
            except Exception as e:
                ipa_transcription = ''
                print(e)
                print(espeak.list_languages())
        else:
            ipa_transcription = Strng

        ipa_transcription = self.IPAFixer(ipa_transcription)

        # introduce Aspiration if selected
        if 'AspirateStops' in self.preoptions:
            ipa_transcription = self.AspirateStops(ipa_transcription)
            self.preoptions.remove('AspirateStops')

        # use explicit dental t,d for romance languages
        if 'es' in self.src or 'fr' in self.src or 'pt' in self.src or 'it' in self.src:
            ipa_transcription = re.sub('t(?![ʃs])', 't̪', ipa_transcription)
            ipa_transcription = re.sub('d(?![ʒz])', 'd̪', ipa_transcription)

        # remove stress by default
        if 'showstress' not in self.postoptions:
            ipa_transcription = ipa_transcription.replace('ˈ', '').replace('ˌ', '')
            #self.preoptions.remove('showstress')

        if 'IPA' in self.tgt:
            return ipa_transcription
        else:
            transliterator = tr.Transliterator('IPA', self.tgt, \
                                            self.preoptions, self.postoptions)

            tgt_transcription = transliterator.convert(ipa_transcription)

        return tgt_transcription

    def AspirateStops(self, Strng):
        c = '|'.join(helper.CrunchSymbols(helper.Consonants,'IPA'))
        v = '|'.join(helper.CrunchSymbols(helper.Vowels,'IPA'))

        Strng = re.sub('(\s)([pkt])(?=ˈ('+v+'))', r'\1\2'+'ʰ', Strng)
        Strng = re.sub('^([pkt])(?=ˈ('+v+'))', r'\1'+'ʰ', Strng)
        Strng = re.sub('('+v+')([pkt])(?=ˈ('+v+'))', r'\1\2'+'ʰ', Strng)

        return Strng

    def IPAFixer(self, Strng):
        Strng = Strng.replace('ᵻ', 'ɨ')
        c = '|'.join(helper.CrunchSymbols(helper.Consonants,'IPA'))

        #duplicate consonants p: => pp
        Strng = re.sub('(' + c + ')' +'(ː)', r'\1\1', Strng)


        return Strng


# -*- coding: utf-8 -*-

### Introduce Nasal to Anusvara for scripts lacking nasal letters but having Anusvara/Chandrabindu

import importlib, string
import re
from functools import reduce

# Crunch Symbols

# Sort Functions
def lenSort(x,y):
    if(len(x[0]) > len(y[0])):
        return -1
    else:
        return 0

def ScriptPath(Script):
    if Script in IndicScripts:
        return 'uchcharaka.ScriptMap.MainIndic.'+Script
    elif Script in LatinScripts:
        return 'uchcharaka.ScriptMap.Roman.'+Script

def retCharList(charList):
    return globals()[charList]

def CrunchSymbols(Part,Script):
    ModScript = importlib.import_module(ScriptPath(Script))
    return reduce(lambda x,y : x+y,[getattr(ModScript,Var) for Var in Part])

def CrunchList(List,Script):
    try:
      ModScript = importlib.import_module(ScriptPath(Script))
    except:
      import logging
      logging.exception('The script ' + Script + ' cannot be found')
      return ''

    return getattr(ModScript,List)

#Introduce in all Latin Conversions
def EscapeChar(Strng):
    punct = "".join(['\\'+x for x in string.punctuation])

    return re.sub('('+punct+')',r'\\'+r'\1',Strng)

# Collection of Symbols
VedicSvaras = '('+ '|'.join(['᳚', '॑', '॒']) + ')?'
VedicSvarasList = ['᳚', '॑', '॒']

Vowels = ['VowelMap','SouthVowelMap','ModernVowelMap','SinhalaVowelMap', 'IPAVowelMap']
VowelSignsNV = ['VowelSignMap','SouthVowelSignMap','ModernVowelSignMap','SinhalaVowelSignMap', 'IPAVowelSignMap']
VowelSigns = ['ViramaMap','VowelSignMap','SouthVowelSignMap','ModernVowelSignMap','SinhalaVowelSignMap', 'IPAVowelSignMap']
CombiningSigns = ['AyogavahaMap','NuktaMap']
Consonants = ['ConsonantMap','SouthConsonantMap','NuktaConsonantMap','SinhalaConsonantMap', 'IPAConsonantMap']

Signs = ['SignMap']
Numerals = ['NumeralMap']
Aytham =['Aytham']
om = ['OmMap']
virama = ['ViramaMap']

Inter = "Inter"

Characters = Vowels + VowelSigns + CombiningSigns + Consonants
CharactersNV = Vowels + VowelSignsNV + CombiningSigns + Consonants

TamilDiacritics = ['ˢʰ', 'ᵖʰ', 'ᵍʰ', 'ᵏʰ', 'ᶜʰʰ', 'ᵗʰ', 'ᵀʰ', 'ᶜʰ', 'ᵈʰ', 'ᵇʰ', 'ᵍ', 'ᵇ', 'ᵈ', 'ᶻ', 'ʷ', 'ᵏ', 'ᵗ','ʰ', 'ᶜ', 'ᶞ', 'ʳ', 'ᶿ', 'ᶠ', 'ᵀ', 'ᴰ', 'ᵖ', 'ʰ', 'ʶ', 'ʸ', 'ʲ']

Diacritics = ['ʽ', '\u00B7', '\u00B9','\u00B2','\u00B3','\u2074','\u2081','\u2082','\u2083','\u2084', '\u1DDC', '\u1DDA', '\u036d', '\u0369', '\u1DEE', '\u1DE8', '\u1DE3', '\u1DF1', '\u1DEB', '\u1de6', '\u0368\u036A', '\u036D\u036A', '\u1DD9', '\u0368', '\u036F', '\u1DDB', '\u0323', '\u036A'] + TamilDiacritics
DiacriticsRemovable = ['ʼ', 'ˇ', 'ˆ', '˘', '\u00B7']
DiacriticsRemovableTamil = ['ˇ', 'ˆ', '˘', '\u00B7']

ScriptAll = ['Aytham', 'Signs', 'CombiningSigns', 'VowelSigns', 'Vowels', 'Consonants', 'Numerals']

inaccurate_scripts = ['IPA', 'Brahmi', 'Grantha', 'Sharada', 'Urdu', 'Sinhala']

IndicScriptsBase = [
               'Devanagari',
               'Telugu',
               'Kannada',
               'Malayalam',
               'TamilPhonetic',
               'Tamil',
               'Sinhala',
               'Urdu'
               ]

suffixes = ['Low', 'Medium', 'High']
IndicScripts = [f"{script}{suffix}" for script in IndicScriptsBase for suffix in suffixes] + IndicScriptsBase

LatinScripts = ['Inter','IPA']

Gemination =  {
               'Urdu': '\u0651'
              }

def add_additional_chars(script_char_map, file_script):
    pass


# -*- coding: utf-8 -*-

from . import helper
from uchcharaka.ScriptMap.MainIndic import Malayalam,Sinhala
import re

class PostProcessor:
    def __init__(self, src, tgt, preoptions=[], postoptions=[], defaults=True):
        self.src = src
        self.tgt = tgt
        if defaults:
            self.postoptions = self.get_default_postprocesses() + postoptions
        else:
            self.postoptions = postoptions
        self.preoptions = preoptions
        self.defaults = defaults

    def get_default_postprocesses(self):
        Options = []
        if 'Telugu' in self.tgt:
            Options = ['NasalToAnusvara','MToAnusvara']

        elif 'Kannada' in self.tgt:
            Options = ['NasalToAnusvara','MToAnusvara']

        elif 'Malayalam' in self.tgt:
            Options = ['MalayalamAnusvaraNasal', 'MToAnusvara']

        elif 'Tamil' in self.tgt:
            Options = ['TamilNaToNNa']

        elif 'Devanagari' in self.tgt:
             Options = ['NasalToAnusvara', 'removeWordEndVirama']

        if ('Devanagari' in self.tgt or 'Telugu' in self.tgt \
            or 'Kannada' in self.tgt) and ('High' not in self.tgt):
            Options += ['nganjaApprox']

        return Options

    def postprocess(self,Strng):
        try:
            for options in self.postoptions:
                Strng = getattr(self, options)(Strng)
        except AttributeError as e:
            pass
            #print('vinodh', e)

        Strng = self.default_postprocess(Strng)

        # Shifting Vowel Signs and Diacritics
        # க²ா ->  கா²
        Strng = self.ShiftDiacritics(Strng)

        return Strng

    ## This is processing that's done after the post-processing steps
    def default_postprocess(self, Strng):
        Strng = Strng.replace('\u034F', '') ## remove token characters for specialized processing
        return Strng

    #Shift Diacritics after vowel signs
    def ShiftDiacritics(self, Strng):
        VS = '|'.join(helper.CrunchSymbols(helper.VowelSigns,self.tgt))
        Diac = '|'.join(helper.Diacritics)
        VedicSign = ''.join(['॑', '॒', '᳚'])
        chandraAnu = '([' + helper.CrunchList('AyogavahaMap',self.tgt)[0] + \
            helper.CrunchList('AyogavahaMap',self.tgt)[1] + '])'
        Strng = re.sub('('+Diac+')'+'('+VS+')',r'\2\1',Strng)

        if 'TamilPhonetic' in self.tgt:
            Strng = re.sub('(\u033C)([\u1DEE\u1DDC\u036D])',r'\2\1', Strng)
        elif 'Tamil' in self.tgt:
            Strng = Strng.replace( '³்', '்³',)
            VedicSign = ['॑', '॒', '᳚']

            Strng = re.sub('(['+''.join(helper.TamilDiacritics)+'])' + \
                           '(['+''.join(VedicSign)+']+)', r'\2\1', Strng)
        elif 'Malayalam' in self.tgt:
            dot = '(\u0323)'
            #Strng = re.sub(dot + '(\u03dc)', r'\2\1', Strng)
            Strng = re.sub(dot + '(['+VedicSign+']+)', r'\2\1', Strng)
            Strng = re.sub(dot + chandraAnu, r'\2\1', Strng)
            Strng = re.sub('(\u0323)([\u0D3B\u0d3C])'+chandraAnu, r'\2\3\1', Strng)

        elif 'Telugu' in self.tgt or 'Devanagari' in self.tgt or 'Malayalam' in self.tgt:
            Strng = re.sub('(['+VedicSign+']+)'+chandraAnu, r'\2\1', Strng)

        return Strng

    def gdbDifferentiationTamilPhonetic(self, Strng):
        Strng = re.sub('[\u1DDC\u1DEE\u036D]', '', Strng)

        return Strng

    def gdbDifferentiationTamil(self, Strng):
        Strng = re.sub('[ᵏᵗᵖ]', '', Strng)

        return Strng

    def removeDifferentiationTamil(self, Strng):
        Strng = Strng.replace('ஜᷦ', 'ஃஜ').replace('பᷫ', 'ஃப')

        if 'Phonetic' not in self.tgt:
            for dia in helper.Diacritics:
                Strng = Strng.replace(dia, '')

        return Strng

    def removeDifferentiationTamilPhonetic(self, Strng):
        Strng = Strng.replace('ஜᷦ', 'ஃஜ').replace('பᷫ', 'ஃப')

        if 'Phonetic' not in self.tgt:
            for dia in helper.Diacritics:
                Strng = Strng.replace(dia, '')

        return Strng

    def removeNuktaTelugu(self, Strng):
        Strng = Strng.replace('\u0C3C', '')
        return Strng

    def removeWordEndVirama(self, Strng):
        if 'showWordEndVirama' not in self.postoptions:
            vir = '\u094D'

            Strng = self.showFinalSchwa(Strng)

            Strng = re.sub(f'({vir})(?=(\s))', '', Strng)
            Strng = re.sub(f'({vir})$', '', Strng)

            #print('getting called')

        return Strng

    def showFinalSchwa(self, Strng):
        ListC = '('+"|".join(sorted(helper.CrunchSymbols(helper.Consonants,self.tgt))) + ')'
        ListVS = '('+"|".join(sorted(helper.CrunchSymbols(helper.VowelSigns,self.tgt))) + ')'

        Strng = re.sub(f'{ListC}(\s)(?!{ListVS})', r'\1'+'ऽ '+r'\2', Strng)
        Strng = re.sub(f'{ListC}$(?!{ListVS})', r'\1'+'ऽ', Strng)

        return Strng

    def nganjaApprox(self, Strng):
        vir =  helper.CrunchList('ViramaMap', self.tgt)[0]
        nga =  helper.CrunchList('ConsonantMap', self.tgt)[4]
        anu =  helper.CrunchList('AyogavahaMap', self.tgt)[1]
        ga =   helper.CrunchList('ConsonantMap', self.tgt)[2]

        nja = helper.CrunchList('ConsonantMap', self.tgt)[9]
        na = helper.CrunchList('ConsonantMap', self.tgt)[19]
        ya = helper.CrunchList('ConsonantMap', self.tgt)[25]

        if 'Devanagari' in self.src:
            Strng = Strng.replace(nga+vir, anu+ga+vir)
        else:
            Strng = Strng.replace(nga, anu+ga)

        Strng = Strng.replace(nja, na+vir+ya)

        return Strng

    #show Schwa
    def ShowSchwaHindi(self, Strng):
        from . import PreProcess as PreP
        Strng = PreP.RemoveSchwaHindi(Strng, True)
        return Strng

    def KannadaNormalCandrabindu(self, Strng):
        Strng = Strng.replace('\u0C80', '\u0C81')

        return Strng

    def TeluguRemoveNukta(self, Strng):
        Strng = Strng.replace('\u0C3C', '')

        return Strng

    def removeDiacritics(self, Strng):
        diacritics = ['\u0331', '\u0306', '\u0323', '\u035F', '\u0324', '\u035F', '\u0307', '\u0301', '\u0303', '\u0310', '\u0306', '\u0302', '\u0304']

        for dia in diacritics:
            Strng = Strng.replace(dia, '')

        vowelDia = ['а̄', 'ӣ', 'ӯ', 'ӗ']
        vowel = ['\u0430', '\u0438', '\u0443', '\u044D']

        for x, y in zip(vowelDia, vowel):
            Strng = Strng.replace(x, y)

        return Strng

    def DevanagariACandra(self, Strng):
        Strng = Strng.replace('ऍ', 'ॲ')

        return Strng

    def dotReph(self, Strng):
        ListC = '('+"|".join(sorted(helper.CrunchSymbols(helper.Consonants,"Malayalam"))) + ')'

        Strng = re.sub('(?<!്)' + 'ർ' + ListC,'ൎ' + r'\1', Strng)
        Strng = re.sub('(?<!്)' +'ര്' + ListC,'ൎ' + r'\1', Strng)

        return Strng

    def TamilNaToNNa(self, Strng):
        na = helper.CrunchList('ConsonantMap', self.tgt)[19]
        nna = helper.CrunchList('SouthConsonantMap', self.tgt)[3]
        vir = helper.CrunchList('ViramaMap', self.tgt)[0]
        ta = helper.CrunchList('ConsonantMap', self.tgt)[15]

        ListV = '|'.join(helper.CrunchSymbols(helper.Vowels+helper.VowelSigns+helper.Consonants,self.tgt)+[helper.CrunchList('SignMap', self.tgt)[0].replace('(','\(').replace(')','\)')])

        Strng = re.sub('('+ListV+')'+ helper.VedicSvaras + '('+na+')' + '(?!' + vir + ta + ')',r'\1\2'+nna,Strng)
        Strng = re.sub('('+ListV+')'+ helper.VedicSvaras + '('+na+')' + '(?!' + vir + ta + ')',r'\1\2'+nna,Strng)

        Strng = re.sub('(²|³|⁴)'+ helper.VedicSvaras + '('+na+')' + '(?!' + vir + ta + ')',r'\1\2'+nna,Strng)
        Strng = re.sub('(²|³|⁴)'+ helper.VedicSvaras + '('+na+')' + '(?!' + vir + ta + ')',r'\1\2'+nna,Strng)

        #Strng = re.sub('(²|³|⁴)'+'('+na+')',r'\1'+nna,Strng)

        #Strng = re.sub('(\s)(ன)', r'\1' + 'ந', Strng)
        #Strng = re.sub('(\.)(ன)', r'\1' + 'ந', Strng)
        #Strng = re.sub('^ன', 'ந', Strng)

        Strng = re.sub("(?<=ஶ்ரீ)(ன)(?!" + vir + ")", "ந", Strng)

        return Strng

    # കൽന് കത്ല് ക്ഷേത്ര് കൻല് - Check this

    def InsertGeminationSign(self, Strng): #Fix this
        vir = helper.CrunchSymbols(helper.VowelSigns,self.tgt)[0]
        ConUnAsp = [helper.CrunchList('ConsonantMap', self.tgt)[x] for x in [0,2,5,7,10,12,15,17,20,22,4,9,14,19,24,25,26,27,28,29,30,31,32]]
        ConUnAsp = ConUnAsp + helper.CrunchList('SouthConsonantMap',self.tgt) + helper.CrunchList('NuktaConsonantMap',self.tgt)
        ConAsp   = [helper.CrunchList('ConsonantMap', self.tgt)[x] for x in [1,3,6,8,11,13,16,18,21,23]]
        ConOthrs = [helper.CrunchList('ConsonantMap', self.tgt)[x] for x in [0,2,5,7,10,12,15,17,20,22,4,9,14,19,24]]

        Strng = re.sub('('+'|'.join(ConUnAsp)+')'+'('+vir+')'+r'\1',helper.Gemination[self.tgt]+r'\1',Strng)

        for i in range(len(ConAsp)):
            Strng = re.sub('('+ConUnAsp[i]+')'+'('+vir+')'+'('+ConAsp[i]+')',helper.Gemination[self.tgt]+r'\3',Strng)

        return Strng

    def ReverseGeminationSign(self, Strng): #Fix this
        vir = helper.CrunchSymbols(helper.VowelSigns,self.tgt)[0]
        ConUnAsp = [helper.CrunchList('ConsonantMap', self.tgt)[x] for x in [0,2,5,7,10,12,15,17,20,22,4,9,14,19,24,25,26,27,28,29,30,31,32]]
        ConUnAsp = ConUnAsp + helper.CrunchList('SouthConsonantMap',self.tgt) + helper.CrunchList('NuktaConsonantMap',self.tgt)
        ConAsp   = [helper.CrunchList('ConsonantMap', self.tgt)[x] for x in [1,3,6,8,11,13,16,18,21,23]]
        ConOthrs = [helper.CrunchList('ConsonantMap', self.tgt)[x] for x in [0,2,5,7,10,12,15,17,20,22,4,9,14,19,24]]

        if self.tgt == 'Gurmukhi':
            Strng = Strng.replace('ੱਸ਼਼', 'ਸ਼਼੍ਸ਼਼')

        Strng = re.sub('(' + helper.Gemination[self.tgt] + ')' + '('+'|'.join(ConUnAsp)+')', r'\2' + vir + r'\2', Strng)

        for i in range(len(ConAsp)):
            Strng = re.sub('(' + helper.Gemination[self.tgt] + ')' + '(' + ConAsp [i] +')', ConUnAsp[i] + vir + r'\2', Strng)

        return Strng

    def NasalToAnusvara(self, Strng):
        ListN = [helper.CrunchSymbols(helper.Consonants, self.tgt)[x] for x in [4,19, 9,19,14,19,19,24]]
        ListC = [
                '|'.join(helper.CrunchList('ConsonantMap', self.tgt)[0:4]),
                '|'.join(helper.CrunchList('ConsonantMap', self.tgt)[0:4]), # to fix ಚಿನ್ಕ್ವ಼ೆ
                '|'.join(helper.CrunchList('ConsonantMap', self.tgt)[5:9]),
                '|'.join(helper.CrunchList('ConsonantMap', self.tgt)[5:9]), # to fix ಪನ್ಚ್
                '|'.join(helper.CrunchList('ConsonantMap', self.tgt)[10:14]),
                '|'.join(helper.CrunchList('ConsonantMap', self.tgt)[10:14]), # To fix ಐಲನ್ಡ್ -> ಐಲಂಡ್
                '|'.join(helper.CrunchList('ConsonantMap', self.tgt)[15:19]),
                '|'.join(helper.CrunchList('ConsonantMap', self.tgt)[20:24]),
                ]
        ListCAll = '(' + '|'.join(helper.CrunchSymbols(helper.Characters, self.tgt)) + ')'

        vir = helper.CrunchSymbols(helper.VowelSigns,self.tgt)[0]
        Anu = helper.CrunchSymbols(helper.CombiningSigns,self.tgt)[1]

        for i in range(len(ListN)):
            #print '('+ListN[i]+')'+'('+vir+')'+'('+ListC[i]+')'
            Strng = re.sub(ListCAll + helper.VedicSvaras + '(?<!' + vir + ')' + '('+ListN[i]+')' +'('+vir+')'+'('+ListC[i]+')',r'\1\2'+Anu+r'\5',Strng)
            Strng = re.sub(ListCAll + helper.VedicSvaras + '(?<!' + vir + ')' + '('+ListN[i]+')' +'('+vir+')'+'('+ListC[i]+')',r'\1\2'+Anu+r'\5',Strng)

        for svara in helper.VedicSvarasList:
            Strng = Strng.replace(svara + Anu, Anu + svara)

        ## So that candrao/candra + anusvara does not get confused by candrabindu
        ## replace anusvara with candrabindu
        ## टॅं -> टॅँ
        if 'Devanagari' in self.tgt:
            Strng = re.sub('([ऑऍॲॅै])(ं)', r'\1'+'ँ', Strng)
        elif 'Kannada' in self.tgt:
            Strng = re.sub('(' + 'ಂ)('+ helper.VedicSvaras +'?)(ಜ಼' + ')' , r'\3' + 'ನ್' + r'\4', Strng)
        elif 'Telugu' in self.tgt:
            Strng = re.sub('(' + 'ం)('+ helper.VedicSvaras +'?)(జ఼' + ')' , r'\3' + 'న్' + r'\4', Strng)

        return Strng

    def AnusvaraToNasal(self, Strng):
        nukta = helper.CrunchList('NuktaMap', self.tgt)[0]

        ListN = [helper.CrunchSymbols(helper.Consonants, self.tgt)[x] for x in [4,9,14,19,24]]
        ListC = [
                '|'.join(helper.CrunchList('ConsonantMap', self.tgt)[0:4]),
                '|'.join(helper.CrunchList('ConsonantMap', self.tgt)[5:9]),
                '|'.join(helper.CrunchList('ConsonantMap', self.tgt)[10:14]),
                '|'.join(helper.CrunchList('ConsonantMap', self.tgt)[15:19]),
                '|'.join(helper.CrunchList('ConsonantMap', self.tgt)[20:24]),
                ]
        vir = helper.CrunchSymbols(helper.VowelSigns,self.tgt)[0]
        Anu = helper.CrunchSymbols(helper.CombiningSigns,self.tgt)[1]

        for i in range(len(ListN)):
            Strng = re.sub('('+Anu+')'+ helper.VedicSvaras + '('+ListC[i]+')(?!' + nukta + ')',ListN[i]+vir+r'\2\3',Strng)

            if self.tgt == "Tamil":
                Strng = re.sub('(ம்)'+ helper.VedicSvaras + '(ʼ)' + '('+ListC[i]+')',ListN[i]+vir+r'\2\4',Strng)

        return Strng

    def MalayalamAnusvaraNasal(self, Strng):
        ListCAll = '(' + '|'.join(helper.CrunchSymbols(helper.Characters, self.tgt)) + ')'

        ListNNasal = [Malayalam.ConsonantMap[x] for x in [4,9,14,19,24]]
        ListCNasal = [
                '|'.join(Malayalam.ConsonantMap[0:1]),
                '|'.join(Malayalam.ConsonantMap[5:8]),
                '|'.join(Malayalam.ConsonantMap[10:14]),
                '|'.join(Malayalam.ConsonantMap[15:19]),
                '|'.join(Malayalam.ConsonantMap[20:21]),
                ]

        ListNAnu = [Malayalam.ConsonantMap[x] for x in [4,24]]
        ListCAnu = [
                '|'.join(Malayalam.ConsonantMap[1:4]),
                '|'.join(Malayalam.ConsonantMap[21:24]),
                ]

        vir = Malayalam.ViramaMap[0]
        Anu = Malayalam.AyogavahaMap[1]

        Chillus=['\u0D7A','\u0D7B','\u0D7C','\u0D7D','\u0D7E', 'ഩ‍്']

        for i in range(len(ListNNasal)):
            Strng = re.sub('('+Anu+')'+'('+ListCNasal[i]+')',ListNNasal[i]+vir+r'\2',Strng)

        for i in range(len(ListNAnu)):
            Strng = re.sub(ListCAll + helper.VedicSvaras + '(?<!' + vir + ')'+'(?<![' + ".".join(Chillus) + '])(?<!' + vir +')' + '('+ListNAnu[i]+')'+'('+vir+')'+'('+ListCAnu[i]+')',r'\1\2'+Anu+r'\5',Strng)

        return Strng

    ## Check Namna, ramya -> Malayalam; fix
    def MToAnusvara(self, Strng):
        M = helper.CrunchList('ConsonantMap', self.tgt)[24] + helper.CrunchList('ViramaMap',self.tgt)[0]
        vir = helper.CrunchList('ViramaMap',self.tgt)[0]
        Anusvara = helper.CrunchList('AyogavahaMap',self.tgt)[1]
        ListC = '|'.join(helper.CrunchSymbols(helper.Characters, self.tgt))

        Chillus= '|'.join([vir, '\u0D7A','\u0D7B','\u0D7C','\u0D7D','\u0D7E'])

        ListCAll = '(' + '|'.join(helper.CrunchSymbols(helper.Characters, self.tgt)) + ')'

        Strng = re.sub(ListCAll + helper.VedicSvaras + '(?<!' + vir + ')'+'('+M+')'+'(?!'+ListC+')',r'\1\2'+Anusvara,Strng)

        for svara in helper.VedicSvarasList:
            Strng = Strng.replace(svara + Anusvara, Anusvara + svara)

        #Strng = Strng.replace(M,Anusvara)

        return Strng

    def RemoveDiacritics(self, Strng):
        for x in helper.DiacriticsRemovable:
            Strng = Strng.replace(x,'')

        return Strng

    def RemoveDiacriticsTamil(self, Strng):
        for x in helper.DiacriticsRemovableTamil:
            Strng = Strng.replace(x,'')

        return Strng

    def TamilSubScript(self, Strng):
        SuperScript = ['\u00B9', '\u00B2', '\u00B3','\u2074']
        SubScript = ['\u2081','\u2082','\u2083','\u2084']

        for x,y in zip(SuperScript,SubScript):
            Strng = Strng.replace(x,y)

        return Strng

    def SinhalaConjuncts(self, Strng):
        ListC = Sinhala.ConsonantMap + [Sinhala.SouthConsonantMap[0]]
        vir = Sinhala.ViramaMap[0]
        ZWJ ="\u200D"

        conjoining =[(0, 28), (2, 18), (9, 5), (10, 11), (15, 16), (15, 28), (17, 18), (17, 28), (19, 16), (19, 17), (19, 18), (19, 28) ]

        for x, y in conjoining:
            Strng = Strng.replace(ListC[x] + vir + ListC[y], ListC[x] + vir + ZWJ + ListC[y])

        for x in ListC:
            Strng = Strng.replace(ListC[26] + vir + x, ListC[26] + vir + ZWJ + x)

        for x in ListC:
            for y in ListC:
                Strng = Strng.replace(x + vir + y, x + ZWJ + vir + y)

        Strng = Strng.replace('ර‍්‍ය', 'ර්‍ය')

        return Strng

    def UrduAlternateUU(self, Strng):
        Strng = Strng.replace("\\u064F\\u0648","\u0648\u0657")

        return Strng

    def UrduRemoveShortVowels(self, Strng):
        ShortVowels = ['\u0652','\u064E','\u0650','\u064F', '\u0658']

        for vow in ShortVowels:
            Strng = Strng.replace(vow,"")

        return Strng

    def showstress(self, Strng):
        return Strng

